@@ -5,9 +5,9 @@
 #' @template with
 #' @param seed `[integer(1)]`\cr The random seed to use to evaluate the code.
 #' @param .local_envir `[environment]`\cr The environment to use for scoping.
-#' @param .rng_kind `[character(1)]`\cr Kind of (uniform) RNG to use.
-#' @param .rng_normal_kind `[character(1)]`\cr Kind of normal RNG to use.
-#' @param .rng_sample_kind `[character(1)]`\cr Kind of RNG to use for sampling.
+#' @param .rng_kind,.rng_normal_kind,.rng_sample_kind
+#'   `[character(1)]`\cr Kind of RNG to use. Passed as the `kind`,
+#'   `normal.kind`, and `sample.kind` arguments of [RNGkind()].
 #' @examples
 #' # Same random values:
 #' with_preserve_seed(runif(5))
@@ -19,32 +19,41 @@
 #' with_seed(seed, runif(5))
 #' with_seed(seed <- sample.int(.Machine$integer.max, 1L), runif(5))
 #' @export
-with_seed <- function(seed, code, .rng_kind = "default", .rng_normal_kind = "default", .rng_sample_kind = "default") {
+with_seed <- function(seed,
+                      code,
+                      .rng_kind = NULL,
+                      .rng_normal_kind = NULL,
+                      .rng_sample_kind = NULL) {
   force(seed)
-  force(.rng_kind)
-  force(.rng_normal_kind)
-  force(.rng_sample_kind)
+  rng_kind <- list(.rng_kind, .rng_normal_kind, .rng_sample_kind)
+
   with_preserve_seed({
-    set_seed(list(seed = seed, rng_kind = c(.rng_kind, .rng_normal_kind, .rng_sample_kind)))
+    set_seed(list(seed = seed, rng_kind = rng_kind))
     code
   })
 }
 
 #' @rdname with_seed
 #' @export
-local_seed <- function(seed, .local_envir = parent.frame(), .rng_kind = "default", .rng_normal_kind = "default",
-                       .rng_sample_kind = "default") {
+local_seed <- function(seed,
+                       .local_envir = parent.frame(),
+                       .rng_kind = NULL,
+                       .rng_normal_kind = NULL,
+                       .rng_sample_kind = NULL) {
   old_seed <- get_seed()
-  set_seed(list(seed = seed, rng_kind = c(.rng_kind, .rng_normal_kind, .rng_sample_kind)))
 
-  defer({
+  defer(envir = .local_envir, {
     if (is.null(old_seed)) {
       on.exit(rm_seed(), add = TRUE)
     } else {
       on.exit(set_seed(old_seed), add = TRUE)
     }
-  }, envir = .local_envir)
+  })
 
+  rng_kind <- list(.rng_kind, .rng_normal_kind, .rng_sample_kind)
+  set_seed(list(seed = seed, rng_kind = rng_kind))
+
+  # FIXME
   invisible(seed)
 }
 
@@ -95,24 +104,15 @@ get_seed <- function() {
   )
 }
 
-set_seed <- local({
-  is_before_3.6 <- getRversion() < "3.6"
+set_seed <- function(seed) {
+  restore_rng_kind(seed$rng_kind)
 
-  function(seed) {
-    # Ensure RNGkind() and Normal RNG state is properly reset (cf. #162)
-    if (is_before_3.6) {
-      seed$rng_kind <- seed$rng_kind[1L:2L]
-    }
-
-    do.call(RNGkind, args = as.list(seed$rng_kind))
-
-    if (is.null(seed$seed)) {
-      assign(".Random.seed", seed$random_seed, globalenv())
-    } else {
-      set.seed(seed$seed)
-    }
+  if (is.null(seed$seed)) {
+    assign(".Random.seed", seed$random_seed, globalenv())
+  } else {
+    set.seed(seed$seed)
   }
-})
+}
 
 rm_seed <- function() {
   if (!has_seed()) {
