@@ -69,3 +69,8 @@ test_that("local_tempfile works", {
   expect_false(file.exists(f1))
   expect_false(file.exists(f2))
 })
+
+test_that("local_tempfile() can add data", {
+  path <- local_tempfile(lines = c("a", "b"))
+  expect_equal(readLines(path), c("a", "b"))
+})
